@@ -16,7 +16,11 @@ pkg_caller_call <- function(start = 1) {
 .err <- function(...) {
   chk::err(..., call = pkg_caller_call(start = 2))
 }
-.wrn <- function(...) {
+.wrn <- function(..., immediate = TRUE) {
+  if (immediate && isTRUE(all.equal(getOption("warn"), 0))) {
+    op <- options(warn = 1)
+    on.exit(options(op))
+  }
   chk::wrn(...)
 }
 .msg <- function(...) {
@@ -40,4 +44,13 @@ pkg_caller_call <- function(start = 1) {
                         conditionMessage(e))
              .err(msg, .subclass = "chk_error")
            })
+}
+
+.chk_basic_vector <- function(x, x_name = NULL) {
+  if (is.atomic(x) && is.null(dim(x))) {
+    return(invisible(x))
+  }
+  if (is.null(x_name))
+    x_name <- chk::deparse_backtick_chk((substitute(x)))
+  chk::abort_chk(x_name, " must be an atomic, non-matrix vector", x = x)
 }
\ No newline at end of file
