@@ -9,11 +9,13 @@ allowable.methods <- {c("glm" = "glm", "ps" = "glm",
                         "super" = "super", "superlearner" = "super",
                         "bart" = "bart",
                         "energy" = "energy")}
-method.to.proper.method <- function(method) {
+
+.method_to_proper_method <- function(method) {
   method <- tolower(method)
   unname(allowable.methods[method])
 }
-check.acceptable.method <- function(method, msm = FALSE, force = FALSE) {
+
+.check_acceptable_method <- function(method, msm = FALSE, force = FALSE) {
   bad.method <- FALSE
 
   if (missing(method)) method <- "glm"
@@ -32,14 +34,15 @@ check.acceptable.method <- function(method, msm = FALSE, force = FALSE) {
   }
 
   if (msm && !force && is.character(method)) {
-    m <- method.to.proper.method(method)
+    m <- .method_to_proper_method(method)
     if (m %in% c("nbcbps", "ebal", "ebcw", "optweight", "energy", "kbal")) {
       .err(sprintf("the use of %s with longitudinal treatments has not been validated. Set `weightit.force = TRUE` to bypass this error",
-                   method.to.phrase(m)))
+                   .method_to_phrase(m)))
     }
   }
 }
-check.user.method <- function(method) {
+
+.check_user_method <- function(method) {
   #Check to make sure it accepts treat and covs
   if (all(c("covs", "treat") %in% names(formals(method)))) {
   }
@@ -49,11 +52,12 @@ check.user.method <- function(method) {
     .err("the user-provided function to `method` must contain `covs` and `treat` as named parameters")
   }
 }
-method.to.phrase <- function(method) {
+
+.method_to_phrase <- function(method) {
 
   if (is.function(method)) return("a user-defined method")
 
-  method <- method.to.proper.method(method)
+  method <- .method_to_proper_method(method)
   if (method %in% c("glm")) return("propensity score weighting with GLM")
   if (method %in% c("gbm")) return("propensity score weighting with GBM")
   if (method %in% c("cbps")) return("covariate balancing propensity score weighting")
@@ -69,7 +73,8 @@ method.to.phrase <- function(method) {
 
   "the chosen method of weighting"
 }
-process.estimand <- function(estimand, method, treat.type) {
+
+.process_estimand <- function(estimand, method, treat.type) {
   #Allowable estimands
   AE <- list(
     binary = list(  glm = c("ATT", "ATC", "ATE", "ATO", "ATM", "ATOS")
@@ -110,13 +115,14 @@ process.estimand <- function(estimand, method, treat.type) {
 
   if (estimand %nin% AE[[treat.type]][[method]]) {
     .err(sprintf("%s is not an allowable estimand for %s with %s treatments. Only %s allowed",
-                 add_quotes(estimand), method.to.phrase(method), treat.type,
+                 add_quotes(estimand), .method_to_phrase(method), treat.type,
                  word_list(AE[[treat.type]][[method]], quotes = TRUE, and.or = "and", is.are = TRUE)))
   }
 
   estimand
 }
-check.subclass <- function(method, treat.type) {
+
+.check_subclass <- function(method, treat.type) {
   #Allowable estimands
   AE <- list(
     binary = list(  glm = TRUE
@@ -148,10 +154,11 @@ check.subclass <- function(method, treat.type) {
   if (treat.type != "continuous" && !is.function(method) &&
       !AE[[treat.type]][[method]]) {
     .err(sprintf("subclasses are not compatible with %s with %s treatments",
-                 method.to.phrase(method), treat.type))
+                 .method_to_phrase(method), treat.type))
   }
 }
-process.ps <- function(ps, data = NULL, treat) {
+
+.process_ps <- function(ps, data = NULL, treat) {
   if (is_null(ps)) return(NULL)
 
   if (is.character(ps) && length(ps) == 1L) {
@@ -175,7 +182,8 @@ process.ps <- function(ps, data = NULL, treat) {
 
   ps
 }
-process.focal.and.estimand <- function(focal, estimand, treat, treated = NULL) {
+
+.process_focal_and_estimand <- function(focal, estimand, treat, treated = NULL) {
   reported.estimand <- estimand
 
   if (!has_treat_type(treat)) treat <- assign_treat_type(treat)
@@ -265,7 +273,8 @@ process.focal.and.estimand <- function(focal, estimand, treat, treated = NULL) {
        reported.estimand = reported.estimand,
        treated = if (is.factor(treated)) as.character(treated) else treated)
 }
-process.by <- function(by, data, treat, treat.name = NULL, by.arg = "by") {
+
+.process_by <- function(by, data, treat, treat.name = NULL, by.arg = "by") {
 
   ##Process by
   bad.by <- FALSE
@@ -330,23 +339,23 @@ process.by <- function(by, data, treat, treat.name = NULL, by.arg = "by") {
 
   by.components
 }
-process.moments.int <- function(moments, int, method) {
+
+.process_moments_int <- function(moments, int, method) {
 
   if (is.function(method) || method %in% c("npcbps", "ebal", "cbps", "ipt", "ebcw", "optweight", "energy")) {
     chk::chk_flag(int)
 
     if (is_not_null(moments)) {
-      if (length(moments) != 1 || !is.numeric(moments) ||
-          !check_if_zero(moments - round(moments))) {
-        chk::chk_whole_number(moments)
-        if (method == "energy") {
-          chk::chk_gte(moments, 0)
-        }
-        else if (method %in% c("npcbps", "ebal", "cbps", "ipt", "ebcw", "optweight")) {
-          chk::chk_gt(moments, 0)
-        }
-        moments <- as.integer(moments)
+      chk::chk_whole_number(moments)
+
+      if (method == "energy") {
+        chk::chk_gte(moments, 0)
       }
+      else if (method %in% c("npcbps", "ebal", "cbps", "ipt", "ebcw", "optweight")) {
+        chk::chk_gt(moments, 0)
+      }
+
+      moments <- as.integer(moments)
     }
     else {
       moments <- {
@@ -358,39 +367,54 @@ process.moments.int <- function(moments, int, method) {
   else if (is_not_null(moments) && any(mi0 <- c(as.integer(moments) != 1L, int))) {
     .wrn(sprintf("%s not compatible with %s. Ignoring %s",
                  word_list(c("moments", "int")[mi0], and.or = "and", is.are = TRUE, quotes = "`"),
-                 method.to.phrase(method),
+                 .method_to_phrase(method),
                  word_list(c("moments", "int")[mi0], and.or = "and", quotes = "`")))
     moments <- NULL
     int <- FALSE
   }
+
   moments <- as.integer(moments)
 
   list(moments = moments, int = int)
 }
-process.MSM.method <- function(is.MSM.method, method) {
-  methods.with.MSM <- c("optweight")
+
+.process_MSM_method <- function(is.MSM.method, method) {
+  methods.with.MSM <- c("optweight", "cbps")
+
   if (is.function(method)) {
-    if (isTRUE(is.MSM.method)) .err("currently, only user-defined methods that work with `is.MSM.method = FALSE` are allowed")
-    is.MSM.method <- FALSE
-  }
-  else if (method %in% methods.with.MSM) {
-    if (is_null(is.MSM.method)) is.MSM.method <- TRUE
-    else if (!isTRUE(is.MSM.method)) {
-      .msg(paste0("%s can be used with a single model when multiple time points are present.\nUsing a seperate model for each time point. To use a single model, set `is.MSM.method` to `TRUE`",
-                  method.to.phrase(method)))
-    }
-  }
-  else {
     if (isTRUE(is.MSM.method)) {
-      .wrn(sprintf("%s cannot be used with a single model when multiple time points are present.\nUsing a seperate model for each time point",
-                   method.to.phrase(method)))
+      .err("currently, only user-defined methods that work with `is.MSM.method = FALSE` are allowed")
     }
-    is.MSM.method <- FALSE
+
+    return(FALSE)
   }
 
-  is.MSM.method
+  if (method %in% methods.with.MSM) {
+    if (is_null(is.MSM.method)) return(TRUE)
+
+    chk::chk_flag(is.MSM.method)
+
+    if (!is.MSM.method) {
+      .msg(paste0("%s can be used with a single model when multiple time points are present.\nUsing a seperate model for each time point. To use a single model, set `is.MSM.method` to `TRUE`",
+                  .method_to_phrase(method)))
+    }
+
+    return(is.MSM.method)
+  }
+
+  if (is_null(is.MSM.method)) return(FALSE)
+
+  chk::chk_flag(is.MSM.method)
+
+  if (is.MSM.method) {
+    .wrn(sprintf("%s cannot be used with a single model when multiple time points are present.\nUsing a seperate model for each time point",
+                 .method_to_phrase(method)))
+  }
+
+  FALSE
 }
-process.missing <- function(missing, method, treat.type) {
+
+.process_missing <- function(missing, method, treat.type) {
   #Allowable estimands
   AE <- list(binary = list(glm = c("ind", "saem")
                            , gbm = c("ind", "surr")
@@ -441,28 +465,30 @@ process.missing <- function(missing, method, treat.type) {
   }
 
   chk::chk_string(missing)
+
   if (!missing %pin% allowable.missings) {
     .err(sprintf("only %s allowed for `missing` with `method = %s` for %s treatments",
                  word_list(allowable.missings, quotes = 2, is.are = TRUE),
                  add_quotes(method),
                  treat.type))
-    return(allowable.missings[1])
   }
 
   allowable.missings[pmatch(missing, allowable.missings)]
 }
-make.closer.to.1 <- function(x) {
+
+.make_closer_to_1 <- function(x) {
   if (chk::vld_character_or_factor(x) || all_the_same(x)) {
     return(x)
   }
 
   if (is_binary(x)) {
-    return(as.numeric(x == x[!is.na(x)][1]))
+    return(as.numeric(x == max(x, na.rm = TRUE)))
   }
 
   (x - mean_fast(x, TRUE))/sd(x, na.rm = TRUE)
 }
-int.poly.f <- function(d, ex = NULL, int = FALSE, poly = 1, center = TRUE, orthogonal_poly = TRUE) {
+
+.int_poly_f <- function(d, ex = NULL, int = FALSE, poly = 1, center = TRUE, orthogonal_poly = TRUE) {
   #Adds to data frame interactions and polynomial terms
   #d=matrix input
   #ex=names of variables to exclude in interactions and polynomials; a subset of df
@@ -520,7 +546,8 @@ int.poly.f <- function(d, ex = NULL, int = FALSE, poly = 1, center = TRUE, ortho
 
   out
 }
-quantile_f <- function(d, qu = NULL, s.weights = NULL, focal = NULL, treat = NULL, const = 2000) {
+
+.quantile_f <- function(d, qu = NULL, s.weights = NULL, focal = NULL, treat = NULL, const = 2000) {
   # Creates new variables for use in balance quantiles. `qu` is a list of quantiles for each
   # continuous variable in `d`, and returns a matrix with a column for each requested quantile
   # of each variable, taking on 0 for values less than the quantile, .5 for values at the quantile,
@@ -580,6 +607,7 @@ quantile_f <- function(d, qu = NULL, s.weights = NULL, focal = NULL, treat = NUL
     out
   }))
 }
+
 get.s.d.denom.weightit <- function(s.d.denom = NULL, estimand = NULL, weights = NULL, treat = NULL, focal = NULL) {
   check.estimand <- check.weights <- check.focal <- FALSE
   s.d.denom.specified <- is_not_null(s.d.denom)
@@ -643,6 +671,7 @@ get.s.d.denom.weightit <- function(s.d.denom = NULL, estimand = NULL, weights =
 
   s.d.denom
 }
+
 get.s.d.denom.cont.weightit <- function(s.d.denom = NULL) {
   s.d.denom.specified <- is_not_null(s.d.denom)
 
@@ -660,7 +689,8 @@ get.s.d.denom.cont.weightit <- function(s.d.denom = NULL) {
 
   try.s.d.denom
 }
-check_estimated_weights <- function(w, treat, treat.type, s.weights) {
+
+.check_estimated_weights <- function(w, treat, treat.type, s.weights) {
 
   tw <- w * s.weights
 
@@ -707,7 +737,8 @@ check_estimated_weights <- function(w, treat, treat.type, s.weights) {
     .wrn("some weights are negative; these cannot be used in most model fitting functions, including `(g)lm_weightit()`")
   }
 }
-ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL,
+
+.ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL,
                          treated = NULL, estimand = NULL) {
   if (is_(ps, c("matrix", "data.frame"))) {
     ps.names <- rownames(ps)
@@ -786,6 +817,7 @@ ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL,
 
   ps
 }
+
 .subclass_ps_multi <- function(ps_mat, treat, estimand = "ATE", focal = NULL, subclass) {
   chk::chk_count(subclass)
   subclass <- round(subclass)
@@ -815,21 +847,21 @@ ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL,
                                      all.inside = TRUE))
     }
 
-    sub.tab <- table(treat, sub)
+    sub_tab <- table(treat, sub)
 
-    if (any(sub.tab == 0)) {
+    if (any(sub_tab == 0)) {
       # .err("Too many subclasses were requested")
-      sub <- subclass_scoot(sub, treat, ps_mat[,i])
-      sub.tab <- table(treat, sub)
+      sub <- .subclass_scoot(sub, treat, ps_mat[,i])
+      sub_tab <- table(treat, sub)
     }
 
     sub <- as.character(sub)
 
-    sub.totals <- colSums(sub.tab)
-    sub.ps <- setNames(sub.tab[as.character(i), ] / sub.totals,
-                       colnames(sub.tab))
+    sub_totals <- colSums(sub_tab)
+    sub_ps <- setNames(sub_tab[as.character(i), ] / sub_totals,
+                       colnames(sub_tab))
 
-    ps_sub[,i] <- sub.ps[sub]
+    ps_sub[,i] <- sub_ps[sub]
     sub_mat[,i] <- sub
 
     if (ncol(ps_sub) == 2) {
@@ -843,6 +875,7 @@ ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL,
 
   ps_sub
 }
+
 .subclass_ps_bin <- function(ps, treat, estimand = "ATE", subclass) {
   chk::chk_count(subclass)
   subclass <- round(subclass)
@@ -860,25 +893,26 @@ ps_to_ps_mat <- function(ps, treat, assumed.treated = NULL, treat.type = NULL,
                                  all.inside = TRUE))
 
   max_sub <- max(sub)
-  sub.tab1 <- tabulate(sub[treat == 1], max_sub)
-  sub.tab0 <- tabulate(sub[treat == 0], max_sub)
+  sub_tab1 <- tabulate(sub[treat == 1], max_sub)
+  sub_tab0 <- tabulate(sub[treat == 0], max_sub)
 
-  if (any(sub.tab1 == 0) || any(sub.tab0 == 0)) {
-    sub <- subclass_scoot(sub, treat, ps)
-    sub.tab1 <- tabulate(sub[treat == 1], max_sub)
-    sub.tab0 <- tabulate(sub[treat == 0], max_sub)
+  if (any(sub_tab1 == 0) || any(sub_tab0 == 0)) {
+    sub <- .subclass_scoot(sub, treat, ps)
+    sub_tab1 <- tabulate(sub[treat == 1], max_sub)
+    sub_tab0 <- tabulate(sub[treat == 0], max_sub)
   }
 
-  sub.totals <- sub.tab1 + sub.tab0
-  sub1_prop <- sub.tab1/sub.totals
+  sub_totals <- sub_tab1 + sub_tab0
+  sub1_prop <- sub_tab1/sub_totals
 
-  sub.ps <- sub1_prop[sub]
+  sub_ps <- sub1_prop[sub]
 
-  attr(sub.ps, "sub") <- sub
+  attr(sub_ps, "sub") <- sub
 
-  sub.ps
+  sub_ps
 }
-subclass_scoot <- function(sub, treat, x, min.n = 1) {
+
+.subclass_scoot <- function(sub, treat, x, min.n = 1) {
   #Reassigns subclasses so there are no empty subclasses
   #for each treatment group. min.n is the smallest a
   #subclass is allowed to be.
@@ -905,9 +939,11 @@ subclass_scoot <- function(sub, treat, x, min.n = 1) {
     }
   }
 
-  if (any({sub_tab <- table(treat, sub)} == 0)) {
+  sub_tab <- table(treat, sub)
 
-    soft_thresh <- function(x, minus = 1) {
+  if (any(sub_tab == 0)) {
+
+    .soft_thresh <- function(x, minus = 1) {
       x <- x - minus
       x[x < 0] <- 0
       x
@@ -920,8 +956,8 @@ subclass_scoot <- function(sub, treat, x, min.n = 1) {
 
           if (first_0 == nsub ||
               (first_0 != 1 &&
-               sum(soft_thresh(sub_tab[t, seq(1, first_0 - 1)]) / abs(first_0 - seq(1, first_0 - 1))) >=
-               sum(soft_thresh(sub_tab[t, seq(first_0 + 1, nsub)]) / abs(first_0 - seq(first_0 + 1, nsub))))) {
+               sum(.soft_thresh(sub_tab[t, seq(1, first_0 - 1)]) / abs(first_0 - seq(1, first_0 - 1))) >=
+               sum(.soft_thresh(sub_tab[t, seq(first_0 + 1, nsub)]) / abs(first_0 - seq(first_0 + 1, nsub))))) {
             #If there are more and closer nonzero subs to the left...
             first_non0_to_left <- max(seq(1, first_0 - 1)[sub_tab[t, seq(1, first_0 - 1)] > 0])
 
@@ -952,6 +988,7 @@ subclass_scoot <- function(sub, treat, x, min.n = 1) {
 
   sub
 }
+
 stabilize_w <- function(weights, treat) {
   if (is.factor(treat)) t.levels <- levels(treat)
   else t.levels <- unique(treat)
@@ -961,13 +998,14 @@ stabilize_w <- function(weights, treat) {
 
   setNames(weights * tab[as.character(treat)], w.names)
 }
+
 `%+%` <- function(...) {
   if (is.atomic(..1) && is.atomic(..2)) crayon::`%+%`(as.character(..1), as.character(..2))
   else ggplot2::`%+%`(...)
 }
 
-get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel = NULL,
-                         n = NULL, treat = NULL, density = NULL) {
+.get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel = NULL,
+                         n = NULL, treat = NULL, density = NULL, weights = NULL) {
   if (is_null(n)) n <- 10 * length(treat)
   if (is_null(adjust)) adjust <- 1
 
@@ -975,9 +1013,9 @@ get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel =
     if (is_null(bw)) bw <- "nrd0"
     if (is_null(kernel)) kernel <- "gaussian"
 
-    densfun <- function(p, s.weights) {
+    densfun <- function(p) {
       d <- density(p, n = n,
-                   weights = s.weights/sum(s.weights), give.Rkern = FALSE,
+                   weights = weights/sum(weights), give.Rkern = FALSE,
                    bw = bw, adjust = adjust, kernel = kernel)
       out <- with(d, approxfun(x = x, y = y))(p)
       attr(out, "density") <- d
@@ -1006,10 +1044,10 @@ get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel =
     }
     else .err("the argument to `density` cannot be evaluated as a density function")
 
-    densfun <- function(p, s.weights) {
+    densfun <- function(p) {
       # sd <- sd(p)
-      sd <- sqrt(col.w.v(p, s.weights))
-      dens <- density(p/sd)
+      # sd <- sqrt(col.w.v(p, s.weights))
+      dens <- density(p)
       if (is_null(dens) || !is.numeric(dens) || anyNA(dens)) {
         .err("there was a problem with the output of `density`. Try another density function or leave it blank to use the Gaussian density")
       }
@@ -1021,7 +1059,7 @@ get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel =
                    max(p) + 3 * adjust * bw.nrd0(p),
                    length.out = n)
       attr(dens, "density") <- data.frame(x = x,
-                                          y = density(x/sd))
+                                          y = density(x))
       dens
     }
   }
@@ -1147,7 +1185,7 @@ get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel =
 .get_w_from_ps_internal_array <- function(ps, treat, estimand = "ATE", focal = NULL,
                                     subclass = NULL, stabilize = FALSE) {
   #Batch turn PS into weights; primarily for output of predict.gbm
-  # Assumes a (0,1) treatment if binary, with ATT already processed
+  # Assumes a (0,1) treatment if binary
   if (is_null(dim(ps))) {
     ps <- matrix(ps, ncol = 1)
   }
@@ -1165,7 +1203,10 @@ get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel =
       w <- treat/ps + (1-treat)/(1-ps)
     }
     else if (estimand == "ATT") {
-      w <- treat + (1-treat)*ps/(1-ps)
+      w <- treat + (1-treat) * ps/(1-ps)
+    }
+    else if (estimand == "ATC") {
+      w <- treat*(1-ps)/ps + (1-treat)
     }
     else if (estimand == "ATO") {
       w <- ps * (1-ps)
@@ -1196,7 +1237,7 @@ get_dens_fun <- function(use.kernel = FALSE, bw = NULL, adjust = NULL, kernel =
 
     if (estimand == "ATE") {
     }
-    else if (estimand == "ATT") {
+    else if (estimand %in% c("ATT", "ATC")) {
       w <- w * ps[, as.character(focal),]
     }
     else if (estimand == "ATO") {
@@ -1291,13 +1332,6 @@ generalized_inverse <- function(sigma) {
   sigma_inv
 }
 
-#Choleski decomp for non-negative definite matrices
-chol2 <- function(Sinv) {
-  ch <- suppressWarnings(chol(Sinv, pivot = TRUE))
-  p <- order(attr(ch, "pivot"))
-  ch[, p, drop = FALSE]
-}
-
 #Compute gradient numerically using centered difference
 gradient <- function(.f, .x, .eps = 1e-8, ...) {
 
